<?php
/*
 * Form Model
 *
 * @package Joomla.Administrator
 * @subpackage Fabrik
 * @since		1.6
 * @copyright Copyright (C) 2005 Rob Clayburn. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 */

// No direct access.
defined('_JEXEC') or die;

require_once('fabmodeladmin.php');

class FabrikModelForm extends FabModelAdmin
{
	/**
	 * @var		string	The prefix to use with controller messages.
	 * @since	1.6
	 */
	protected $text_prefix = 'COM_FABRIK_FORM';

	protected $abstractPlugins = null;

	protected $pluginType = 'Form';


	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param	type	The table type to instantiate
	 * @param	string	A prefix for the table class name. Optional.
	 * @param	array	Configuration array for model. Optional.
	 * @return	JTable	A database object
	 * @since	1.6
	 */

	public function getTable($type = 'Form', $prefix = 'FabrikTable', $config = array())
	{
		$config['dbo'] = FabriKWorker::getDbo();
		return FabTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param	array	$data		Data for the form.
	 * @param	boolean	$loadData	True if the form is to load its own data (default case), false if not.
	 * @return	mixed	A JForm object on success, false on failure
	 * @since	1.6
	 */

	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_fabrik.form', 'form', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		$form->model = $this;
		return $form;
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return	mixed	The data for the form.
	 * @since	1.6
	 */

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_fabrik.edit.form.data', array());
		if (empty($data)) {
			$data = $this->getItem();
		}
		return $data;
	}

	/**
	 * get the possible list plug-ins that can be selected for use
	 * @return array
	 */

	public function getAbstractPlugins()
	{
		if (isset($this->abstractPlugins)) {
			return $this->abstractPlugins;
		}

		// create a new dispatcher so that we only collect admin html for validation rules
		$pluginDispatcher = new JDispatcher();

		//import the validation plugins and assign them to their custom dispatcher
		JPluginHelper::importPlugin('fabrik_form', null, true, $pluginDispatcher);
		$rules = array();
		//trigger the validation dispatcher to get hte validation rules html

		$plugins = JPluginHelper::getPlugin('fabrik_form');

		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$pluginManager = JModel::getInstance('Pluginmanager', 'FabrikFEModel');

		$feFormModel = JModel::getInstance('form', 'fabrikFEModel');
		$feFormModel->setId($this->getState('form.id'));

		foreach ($plugins as $x => $plugin) {

			$data = array();

			$o = $pluginManager->getPlugIn($plugin->name, 'Form');

			$o->getJForm()->model = $feFormModel;

			// $$$ rob 0 was $x below but that rendered first set of plugins with indexes 1,2,3
			// think they should all be indexed 0
			$str = $o->onRenderAdminSettings($data, 0);
			$js = $o->onGetAdminJs($plugin->name, $plugin->name, $str);
			$str = addslashes(str_replace(array("\n", "\r"), "", $str));

			$attr = "class=\"inputbox elementtype\"";
			$rules[$plugin->name] = array('plugin'=>$plugin->name, 'html'=>$str, 'js'=>$js);
		}
		asort($rules);
		$this->abstractPlugins = $rules;
		return $rules;
	}

	public function getJs()
	{

		$abstractPlugins = $this->getAbstractPlugins();
		$plugins = $this->getPlugins();
		$item = $this->getItem();
		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$pluginManager = JModel::getInstance('Pluginmanager', 'FabrikFEModel');

		JText::script('COM_FABRIK_ACTION');
		JText::script('COM_FABRIK_SELECT_DO');
		JText::script('COM_FABRIK_DELETE');
		JText::script('COM_FABRIK_IN');
		JText::script('COM_FABRIK_ON');
		JText::script('COM_FABRIK_OPTIONS');
		JText::script('COM_FABRIK_PLEASE_SELECT');
		JText::script('COM_FABRIK_FRONT_END');
		JText::script('COM_FABRIK_BACK_END');
		JText::script('COM_FABRIK_BOTH');
		JText::script('COM_FABRIK_NEW');
		JText::script('COM_FABRIK_EDIT');

		$js =
	"
  window.addEvent('load', function(){\n";
		$js .= "\t\tvar aPlugins = [];\n";
		foreach ($abstractPlugins as $abstractPlugin) {
			$js .= "\t\taPlugins.push(".$abstractPlugin['js'].");\n";
		}
		$js .= "controller = new fabrikAdminForm(aPlugins);\n";
		foreach($plugins as $plugin) {
			$js .= "controller.addAction('".$plugin['html']."', '".$plugin['plugin']."', '".@$plugin['location']."', '".@$plugin['event']."', false);\n";
		}
		$js .= "
});";
		return $js;
	}


	protected function getPluginLocation($repeatCounter)
	{
		$item = $this->getItem();
		return $item->params['plugin_locations'][$repeatCounter];
	}

	protected function getPluginEvent($repeatCounter)
	{
		$item = $this->getItem();
		return $item->params['plugin_events'][$repeatCounter];
	}

	/**
	 * save the form
	 * @param array $data
	 */

	function save($data)
	{
		$post = JRequest::get('post');
		$data['params']['plugins'] = (array)JArrayHelper::getValue($post['jform'], 'plugin');
		$data['params']['plugin_locations'] = (array)JArrayHelper::getValue($post['jform'], 'plugin_locations');
		$data['params']['plugin_events'] = (array)JArrayHelper::getValue($post['jform'], 'plugin_events');

		// move back into the main data array some values we are rendering as
		// params (did that for ease of rendering admin output)

		$opts = array('reset_button_label', 'submit_button_label');
		foreach ($opts as $opt) {
			$data[$opt] = $data['params'][$opt];
		}
		$tmpName = JArrayHelper::getValue($data, 'db_table_name');
		unset($data['db_table_name']);

		$return = parent::save($data);
		if ($return) {
			$data['db_table_name'] = $tmpName;
			$this->saveFormGroups($data);
		}
		return $return;
	}

	/**
	 * After having saved the form we
	 * 1) Create a new group if none selected in edit form list
	 * 2) Delete all old form_group records
	 * 3) Recreate the form group records
	 * 4) Make a table view if needed
	 * @return bol true if you should display the form list, false if you're
	 * redirected elsewhere
	 */

	public function saveFormGroups($data)
	{
		//these are set in parent::save() and contain the updated form id and if the form is a new form
		$formid = $this->getState($this->getName().'.id');
		$isnew = $this->getState($this->getName().'.new');

		$db = FabrikWorker::getDbo();
		$currentGroups = $data['current_groups'];
		$record_in_database = $data['record_in_database'];
		$createGroup = $data['_createGroup'];
		$form =& $this->getForm();
		if ($createGroup) {
			$group = FabTable::getInstance('Group', 'FabrikTable');
			$group->name = $data['label'];
			$group->published = 1;
			$group->store();
			$currentGroups[] = $db->insertid();
		}
		$this->_makeFormGroups($data, $currentGroups);

		if ($record_in_database == '1') {
			JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
			$listModel = JModel::getInstance('List', 'FabrikFEModel');
			$listModel->loadFromFormId($data['id']);
			$table =& $listModel->getTable();
			if ($isnew) {
				$dbTableName = $data['db_table_name'];
			} else {
				$dbTableName = $table->db_table_name == '' ? $data['database_name'] : $table->db_table_name;
			}

			// @TODO - sanitize table name, but not sure whether to do it here, or after the databaseTableExists() call.
			//$dbTableName = preg_replace('#[^0-9a-zA-Z_]#', '_', $dbTableName);

			// @TODO - allow user to select connection when saving form

			$defaultDb = $listModel->getDb();

			$formModel = JModel::getInstance('Form', 'FabrikFEModel');

			$formModel->setId($formid);

			if (!$listModel->databaseTableExists($dbTableName, $defaultDb)) {
				// @TODO - need to sanitize table name (get rid of non alphanumeirc or _),
				// just not sure whether to do it here, or above (before we test for existinance)
				// $$$ hugh - for now, just do it here, after we test for the 'unsanitized', as
				// need to do some more testing on MySQL table name case sensitivity
				// BUT ... as we're potentially changing the table name after testing for existance
				// we need to test again.
				$dbTableName = preg_replace('#[^0-9a-zA-Z_]#', '_', $dbTableName);
				if ($listModel->databaseTableExists($dbTableName, $defaultDb)) {
					return JError::raiseWarning(500, JText::_("COM_FABRIK_DB_TABLE_ALREADY_EXISTS"));
				}
				//need to pass the correct database obj here
				$listModel->createDBTable($formModel, $dbTableName, $defaultDb);

				$connection =& $listModel->getConnection();
				$table->id = null;
				$table->label 				= $data['label'];
				$table->form_id 			= $formid;
				$table->connection_id = $connection->getConnection()->id;
				$table->db_table_name	= $dbTableName;
				// store key without nameQuotes as thats db specific *which we no longer want
				$table->db_primary_key = $dbTableName.'.id';
				$table->auto_inc 			= 1;
				$table->published 		= $data['published'];
				$table->created				= $data['created'];
				$table->created_by		= $data['created_by'];
				$table->params = $listModel->getDefaultParams();
				$res = $table->store();
				if ($res) {
					$listModel->createCacheQuery();
				}
			} else {
				//update existing table
				$listModel->ammendTable($formModel, $dbTableName);
				// $$$ rob no longer in front end model?
				//$listModel->makeSafeTableColumns();
			}
		}
	}

	/**
	 * reinsert the groups ids into formgroup rows
	 * @param array jform post data
	 * @param array $currentGroups
	 */

	protected function _makeFormGroups($data, $currentGroups)
	{
		$formid = $this->getState($this->getName().'.id');
		$db =& FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$query->delete('#__{package}_formgroup')->where("form_id = " . (int)$formid);
		$db->setQuery($query);
		// delete the old form groups
		if (!$db->query()) {
			JError::raiseError(500, $db->stderr());
		}

		$orderid = 1;
		$currentGroups = array_unique($currentGroups);
		foreach ($currentGroups as $group_id) {
			if ($group_id != '') {
				$group_id = (int)$group_id;
				$query = $db->getQuery(true);
				$query->insert('#__{package}_formgroup')->set(array('form_id ='.(int)$formid, 'group_id = '.$group_id, 'ordering = '.$orderid));
				//$sql = "INSERT INTO #__{package}_formgroup (form_id, group_id, ordering) VALUES (" . (int)$formid . ",$group_id, $orderid)";
				$db->setQuery($query);
				if (!$db->query()) {
					JError::raiseError(500, $db->stderr());
				}
				$orderid ++;
			}
		}
	}

	/**
	 * take an array of list ids and return the corresponding form_id's
	 * used in list publish code
	 * @param array list ids
	 * @return array form ids
	 */

	public function swapListToFormIds($ids = array())
	{
		if (empty($ids)) {
			return array();
		}
		JArrayHelper::toInteger($ids);
		$db = FabrikWorker::getDbo();
		$query = $db->getQuery(true);
		$query->select('form_id')->from('#__{package}_lists')->where('id IN ('. implode(',', $ids).')');
		echo $query;
		return $db->setQuery($query)->loadResultArray();
	}

	/**
	 * iterate over the form's elements and update its db table to match
	 */

	public function updateDatabase()
	{
		$db = FabrikWorker::getDbo();
		$cid		= JRequest::getVar('cid', null, 'post', 'array');
		$formId = $cid[0];
		JModel::addIncludePath(JPATH_SITE.DS.'components'.DS.'com_fabrik'.DS.'models');
		$model = JModel::getInstance('Form', 'FabrikFEModel');
		$model->setId($formId);
		$form = $model->getForm();
		//use this in case there is not table view linked to the form
		if ($form->record_in_database == 1) {

			//there is a table view linked to the form so lets load it
			$listModel  = JModel::getInstance('List', 'FabrikFEModel');
			$listModel->loadFromFormId($formId);
			$dbExisits = $listModel->databaseTableExists();
			if (!$dbExisits) {
				$listModel->createDBTable($model);
			} else {
				$listModel->ammendTable();
			}
		}
	}

	/**
	 * Method to validate the form data.
	 *
	 * @param	object		$form		The form to validate against.
	 * @param	array		$data		The data to validate.
	 * @return	mixed		Array of filtered data if valid, false otherwise.
	 * @since	1.1
	 */
	function validate($form, $data)
	{
		$params = $data['params'];
		$ok = parent::validate($form, $data);
		//standard jform validation failed so we shouldn't test further as we can't
		//be sure of the data
		if (!$ok) {
			return false;
		}
		//hack - must be able to add the plugin xml fields file to $form to include in validation
		// but cant see how at the moment
		$data['params'] = $params;
		return $data;
	}

}
?>